@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -524,10 +524,8 @@ FileIO_CloseAndUnlink(FileIODescriptor *fd)  // IN:
    path = Unicode_Duplicate(fd->fileName);
 
    ret = FileIO_Close(fd);
-   if (FileIO_IsSuccess(ret)) {
-      if (File_UnlinkIfExists(path) == -1) {
-         ret = FILEIO_ERROR;
-      }
+   if ((File_UnlinkIfExists(path) != 0) && FileIO_IsSuccess(ret)) {
+      ret = FILEIO_ERROR;
    }
 
    Posix_Free(path);
@@ -697,7 +695,7 @@ FileIOResult
 FileIO_AtomicTempFile(FileIODescriptor *fileFD,  // IN:
                       FileIODescriptor *tempFD)  // OUT:
 {
-   char *tempPath = NULL;
+   char *tempPath;
    int permissions;
    FileIOResult status;
 #if !defined(_WIN32)
@@ -838,8 +836,6 @@ FileIO_AtomicUpdateEx(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
    uint32 newAccess;
    FileIOResult status;
    FileIODescriptor tmpFD;
-#else
-   int fd;
 #endif
    int savedErrno = 0;
    int ret = 0;
@@ -849,11 +845,12 @@ FileIO_AtomicUpdateEx(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
 
    if (HostType_OSIsVMK()) {
 #if defined(VMX86_SERVER)
-      FS_SwapFilesArgsUW *args = NULL;
+      FS_SwapFilesArgsUW args = { 0 };
       char *dirName = NULL;
       char *fileName = NULL;
       char *dstDirName = NULL;
       char *dstFileName = NULL;
+      Bool isSame;
 
       currPath = File_FullPath(FileIO_Filename(currFD));
       if (currPath == NULL) {
@@ -875,14 +872,23 @@ FileIO_AtomicUpdateEx(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
       File_GetPathName(currPath, &dstDirName, &dstFileName);
 
       ASSERT(dirName != NULL);
-      ASSERT(fileName && *fileName);
+      ASSERT(fileName != NULL && *fileName != '\0');
       ASSERT(dstDirName != NULL);
-      ASSERT(dstFileName && *dstFileName);
-      ASSERT(File_IsSameFile(dirName, dstDirName));
+      ASSERT(dstFileName != NULL && *dstFileName != '\0');
 
-      args = Util_SafeCalloc(1, sizeof *args);
-      args->fd = currFD->posix;
-      if (ioctl(newFD->posix, IOCTLCMD_VMFS_SWAP_FILES, args) != 0) {
+      errno = 0;
+      isSame = File_IsSameFile(dirName, dstDirName);
+      if (errno == 0) {
+         ASSERT(isSame);
+      } else {
+         savedErrno = errno;
+         Log("%s: File_IsSameFile of ('%s', '%s') failed: %d\n", __FUNCTION__,
+             dirName, dstDirName, errno);
+         goto swapdone;
+      }
+
+      args.fd = currFD->posix;
+      if (ioctl(newFD->posix, IOCTLCMD_VMFS_SWAP_FILES, &args) != 0) {
          savedErrno = errno;
          if (errno != ENOSYS && errno != ENOTTY) {
             Log("%s: ioctl failed %d.\n", __FUNCTION__, errno);
@@ -901,6 +907,8 @@ FileIO_AtomicUpdateEx(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
        */
       if (savedErrno == ENOSYS || savedErrno == ENOTTY) {
          if (renameOnNFS) {
+            int fd;
+
             /*
              * NFS allows renames of locked files, even if both files
              * are locked.  The file lock follows the file handle, not
@@ -932,7 +940,6 @@ FileIO_AtomicUpdateEx(FileIODescriptor *newFD,   // IN/OUT: file IO descriptor
       }
 
 swapdone:
-      Posix_Free(args);
       Posix_Free(dirName);
       Posix_Free(fileName);
       Posix_Free(dstDirName);
@@ -1008,6 +1015,8 @@ swapdone:
           __FUNCTION__, newPath, currPath, errno);
           savedErrno = errno;
    } else {
+      int fd;
+
       ret = TRUE;
       fd = newFD->posix;
       newFD->posix = currFD->posix;
@@ -1027,7 +1036,7 @@ swapdone:
  *
  * FileIO_AtomicUpdate --
  *
- *      Wrapper around FileIO_AtomicUpdateEx that derfaults 'renameOnNFS' to
+ *      Wrapper around FileIO_AtomicUpdateEx that defaults 'renameOnNFS' to
  *      TRUE.
  *
  * Results:
