@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -135,6 +135,10 @@ static const int FileIO_OpenActions[] = {
    O_CREAT | O_TRUNC,
 };
 
+#ifdef __APPLE__
+static FileIOPrivilegedOpener *privilegedOpenerFunc = NULL;
+#endif
+
 /*
  * Options for FileCoalescing performance optimization
  */
@@ -207,7 +211,7 @@ static AlignedPool alignedPool;
       extern ssize_t pwritev64(int fd, const struct iovec *iov, int iovcnt,
                           __off64_t offset) __attribute__ ((weak));
    #else
-      #error "Large file support unavailable. Aborting."
+      #error "Large file support is unavailable."
    #endif
 #endif /* defined(__linux__) */
 
@@ -453,67 +457,15 @@ FileIO_CreateFDPosix(int posix,  // IN: UNIX file descriptor
 /*
  *----------------------------------------------------------------------
  *
- * ProxySendResults --
+ * FileIO_SetPrivilegedOpener --
  *
- *      Send the results of a open from the proxy.
+ *      Set the function to be used when opening files with privilege,
+ *      overriding the default behavior. See FileIO_PrivilegedPosixOpen.
  *
- * Results:
- *	None.
+ *      Setting the privileged opener to NULL will restore default
+ *      behavior.
  *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-static void
-ProxySendResults(int sock_fd,     // IN:
-                 int send_fd,     // IN:
-                 int send_errno)  // IN:
-{
-   struct iovec iov;
-   struct msghdr msg;
-   char cmsgBuf[CMSG_SPACE(sizeof send_fd)];
-
-   iov.iov_base = &send_errno;
-   iov.iov_len = sizeof send_errno;
-
-   if (send_fd == -1) {
-      msg.msg_control = NULL;
-      msg.msg_controllen = 0;
-   } else {
-      struct cmsghdr *cmsg;
-
-      msg.msg_control = cmsgBuf;
-      msg.msg_controllen = sizeof cmsgBuf;
-
-      cmsg = CMSG_FIRSTHDR(&msg);
-
-      cmsg->cmsg_level = SOL_SOCKET;
-      cmsg->cmsg_len = CMSG_LEN(sizeof send_fd);
-      cmsg->cmsg_type = SCM_RIGHTS;
-
-      (*(int *) CMSG_DATA(cmsg)) = send_fd;
-
-      msg.msg_controllen = cmsg->cmsg_len;
-   }
-
-   msg.msg_name = NULL;
-   msg.msg_namelen = 0;
-   msg.msg_iov = &iov;
-   msg.msg_iovlen = 1;
-   msg.msg_flags = 0;
-
-   sendmsg(sock_fd, &msg, 0);
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * ProxyReceiveResults --
- *
- *      Receive the results of an open from the proxy.
+ *      This function is not thread safe.
  *
  * Results:
  *      None
@@ -524,253 +476,11 @@ ProxySendResults(int sock_fd,     // IN:
  *----------------------------------------------------------------------
  */
 
-static void
-ProxyReceiveResults(int sock_fd,      // IN:
-                    int *recv_fd,     // OUT:
-                    int *recv_errno)  // OUT:
+void
+FileIO_SetPrivilegedOpener(FileIOPrivilegedOpener *opener) // IN
 {
-   int err;
-   struct iovec iov;
-   struct msghdr msg;
-   struct cmsghdr *cmsg;
-   uint8_t cmsgBuf[CMSG_SPACE(sizeof(int))];
-
-   iov.iov_base = recv_errno;
-   iov.iov_len = sizeof *recv_errno;
-
-   msg.msg_control = cmsgBuf;
-   msg.msg_controllen = sizeof cmsgBuf;
-   msg.msg_name = NULL;
-   msg.msg_namelen = 0;
-   msg.msg_iov = &iov;
-   msg.msg_iovlen = 1;
-
-   err = recvmsg(sock_fd, &msg, 0);
-
-   if (err <= 0) {
-      *recv_fd = -1;
-      *recv_errno = (err == 0) ? EIO : errno;
-
-      return;
-   }
-
-   if (msg.msg_controllen == 0) {
-      *recv_fd = -1;
-   } else {
-      cmsg = CMSG_FIRSTHDR(&msg);
-
-      if ((cmsg->cmsg_level == SOL_SOCKET) &&
-          (cmsg->cmsg_type == SCM_RIGHTS)) {
-         *recv_fd = *((int *) CMSG_DATA(cmsg));
-      } else {
-         *recv_fd = -1;
-         *recv_errno = EIO;
-      }
-   }
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * ProxyOpen --
- *
- *      Open a file via a proxy.
- *
- * Results:
- *      -1 on error
- *      >= 0 on success
- *
- * Side effects:
- *      errno is set on error
- *
- *----------------------------------------------------------------------
- */
-
-static int
-ProxyOpen(const char *pathName,  // IN:
-          int flags,             // IN:
-          int mode)              // IN:
-{
-   int err;
-   pid_t pid;
-   int fds[2];
-   int proxyFD;
-
-   int saveErrno = 0;
-
-   if (pathName == NULL) {
-      errno = EFAULT;
-      return -1;
-   }
-
-   err = socketpair(AF_UNIX, SOCK_DGRAM, 0, fds);
-   if (err == -1) {
-      errno = ENOMEM; // Out of resources...
-      return err;
-   }
-
-   pid = fork();
-   if (pid == -1) {
-      proxyFD = -1;
-      saveErrno = ENOMEM; // Out of resources...
-      goto bail;
-   }
-
-   if (pid == 0) { /* child:  use fd[0] */
-      proxyFD = Posix_Open(pathName, flags, mode);
-
-      ProxySendResults(fds[0], proxyFD, errno);
-
-      _exit(0);
-   } else {        /* parent: use fd[1] */
-      ProxyReceiveResults(fds[1], &proxyFD, &saveErrno);
-
-      waitpid(pid, &err, 0);
-   }
-
-bail:
-
-   close(fds[0]);
-   close(fds[1]);
-
-   errno = saveErrno;
-
-   return proxyFD;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * ProxyUse --
- *
- *      Determine is the open proxy is to be used.
- *
- * Results:
- *	0	Success, useProxy is set
- *	> 0	Failure (errno value); useProxy is undefined
- *
- * Side effects:
- *	None
- *
- *----------------------------------------------------------------------
- */
-
-static int
-ProxyUse(const char *pathName,  // IN:
-         Bool *useProxy)        // IN:
-{
-   char *path;
-   UnicodeIndex index;
-   struct statfs sfbuf;
-   struct stat statbuf;
-
-   if (pathName == NULL) {
-      errno = EFAULT;
-      return -1;
-   }
-
-   if ((Posix_Lstat(pathName, &statbuf) == 0) &&
-       S_ISLNK(statbuf.st_mode)) {
-      *useProxy = TRUE;
-
-      return 0;
-   }
-
-   /*
-    * Construct the path to the directory that contains the filePath.
-    */
-
-   index = Unicode_FindLast(pathName, "/");
-
-   if (index == UNICODE_INDEX_NOT_FOUND) {
-      path = Unicode_Duplicate(".");
-   } else {
-      char *temp;
-
-      temp = Unicode_Substr(pathName, 0, index + 1);
-      path = Unicode_Append(temp, ".");
-      Posix_Free(temp);
-   }
-
-   /*
-    * Attempt to obtain information about the testPath (directory
-    * containing filePath).
-    */
-
-   if (Posix_Statfs(path, &sfbuf) == 0) {
-      /*
-       * The testPath exists; determine proxy usage explicitely.
-       */
-
-      *useProxy = strcmp(sfbuf.f_fstypename, "nfs") == 0 ?  TRUE : FALSE;
-   } else {
-      /*
-       * A statfs error of some sort; Err on the side of caution.
-       */
-
-      *useProxy = TRUE;
-   }
-
-   Posix_Free(path);
-
-   return 0;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * PosixFileOpener --
- *
- *      Open a file. Use a proxy when creating a file or on NFS.
- *
- *      Why a proxy? The Mac OS X 10.4.* NFS client interacts with our
- *      use of settid() and doesn't send the proper credentials on opens.
- *      This leads to files being written without error but containing no
- *      data. The proxy avoids all of this unhappiness.
- *
- * Results:
- *      -1 on error
- *      >= 0 on success
- *
- * Side effects:
- *      errno is set
- *
- *----------------------------------------------------------------------
- */
-
-int
-PosixFileOpener(const char *pathName,  // IN:
-                int flags,             // IN:
-                mode_t mode)           // IN:
-{
-   Bool useProxy;
-
-   if ((flags & O_ACCMODE) || (flags & O_CREAT)) {
-      int err;
-
-      /*
-       * Open for write and/or O_CREAT. Determine proxy usage.
-       */
-
-      err = ProxyUse(pathName, &useProxy);
-      if (err != 0) {
-         errno = err;
-
-         return -1;
-      }
-   } else {
-      /*
-       * No write access, no need for a proxy.
-       */
-
-      useProxy = FALSE;
-   }
-
-   return useProxy ? ProxyOpen(pathName, flags, mode) :
-                     Posix_Open(pathName, flags, mode);
+   ASSERT(privilegedOpenerFunc == NULL || opener == NULL);
+   privilegedOpenerFunc = opener;
 }
 #endif
 
@@ -803,7 +513,6 @@ FileIOCreateRetry(FileIODescriptor *file,   // OUT:
                   int mode,                 // IN: mode_t for creation
                   uint32 maxWaitTimeMsec)   // IN: Ignored
 {
-   uid_t uid = -1;
    int fd = -1;
    int flags = 0;
    int error;
@@ -883,8 +592,8 @@ FileIOCreateRetry(FileIODescriptor *file,   // OUT:
       flags |= O_DIRECT;
 #elif !defined(__APPLE__) // Mac hosts need this access flag after opening.
       access &= ~FILEIO_OPEN_UNBUFFERED;
-      LOG_ONCE((LGPFX" %s reverting to buffered IO on %s.\n",
-                __FUNCTION__, pathName));
+      LOG_ONCE(LGPFX" %s reverting to buffered IO on %s.\n",
+               __FUNCTION__, pathName);
 #endif
    }
 
@@ -915,7 +624,7 @@ FileIOCreateRetry(FileIODescriptor *file,   // OUT:
 #endif
 
 #if defined(__linux__) && defined(O_CLOEXEC)
-   if (flags & FILEIO_OPEN_CLOSE_ON_EXEC) {
+   if (access & FILEIO_OPEN_CLOSE_ON_EXEC) {
       flags |= O_CLOEXEC;
    }
 #endif
@@ -924,19 +633,37 @@ FileIOCreateRetry(FileIODescriptor *file,   // OUT:
 
    file->flags = access;
 
+#if defined(__APPLE__)
    if (access & FILEIO_OPEN_PRIVILEGED) {
-      uid = Id_BeginSuperUser();
+      // We only support privileged opens, not creates or truncations.
+      if ((flags & (O_CREAT | O_TRUNC)) != 0) {
+         fd = -1;
+         errno = EACCES;
+      } else {
+         fd = FileIO_PrivilegedPosixOpen(pathName, flags);
+      }
+   } else {
+      fd = Posix_Open(pathName, flags, mode);
    }
+#else
+   {
+      uid_t uid = -1;
 
-   fd = PosixFileOpener(pathName, flags, mode);
+      if (access & FILEIO_OPEN_PRIVILEGED) {
+         uid = Id_BeginSuperUser();
+      }
 
-   error = errno;
+      fd = Posix_Open(pathName, flags, mode);
 
-   if (access & FILEIO_OPEN_PRIVILEGED) {
-      Id_EndSuperUser(uid);
+      error = errno;
+
+      if (access & FILEIO_OPEN_PRIVILEGED) {
+         Id_EndSuperUser(uid);
+      }
+
+      errno = error;
    }
-
-   errno = error;
+#endif
 
    if (fd == -1) {
       ret = FileIOErrno2Result(errno);
@@ -1484,8 +1211,8 @@ FileIOCoalesce(
    }
 
    // XXX: Wouldn't it be nice if we could log from here!
-   //LOG(5, ("FILE: Coalescing %s of %d elements and %d size\n",
-   //        isWrite ? "write" : "read", inCount, inTotalSize));
+   //LOG(5, "FILE: Coalescing %s of %d elements and %d size\n",
+   //    isWrite ? "write" : "read", inCount, inTotalSize);
 
    if (filePosixOptions.aligned || flags & FILEIO_OPEN_UNBUFFERED) {
       cBuf = FileIOAligned_Malloc(sizeof(uint8) * inTotalSize);
@@ -1921,8 +1648,8 @@ FileIOPwritevCoalesced(
              * already checked for retval == -1 above, so the cast
              * below should be OK. Refer to bug 817761.
              */
-            LOG_ONCE((LGPFX" %s wrote %"FMTSZ"u out of %"FMTSZ"u bytes.\n",
-                      __FUNCTION__, (size_t)retval, leftToWrite));
+            LOG_ONCE(LGPFX" %s wrote %"FMTSZ"u out of %"FMTSZ"u bytes.\n",
+                     __FUNCTION__, (size_t)retval, leftToWrite);
          }
 
          buf += retval;
@@ -2008,6 +1735,12 @@ FileIOPreadvInternal(
       ssize_t retval = 0;
 
       ASSERT(numVec > 0);
+
+      /*
+       * This is needed to deal with old libraries.  Once we're over
+       * the library horizon this can go away.
+       */
+      /* coverity[func_conv] */
       if (preadv64 == NULL) {
          fret = FileIOPreadvCoalesced(fd, entries, numEntries, offset,
                                       totalSize, &bytesRead);
@@ -2143,6 +1876,12 @@ FileIOPwritevInternal(
       ssize_t retval = 0;
 
       ASSERT(numVec > 0);
+
+      /*
+       * This is needed to deal with old libraries.  Once we're over
+       * the library horizon this can go away.
+       */
+      /* coverity[func_conv] */
       if (pwritev64 == NULL) {
          fret = FileIOPwritevCoalesced(fd, entries, numEntries, offset,
                                        totalSize, &bytesWritten);
@@ -2648,8 +2387,6 @@ FileIO_PrivilegedPosixOpen(const char *pathName,  // IN:
                            int flags)             // IN:
 {
    int fd;
-   Bool suDone;
-   uid_t uid = -1;
 
    if (pathName == NULL) {
       errno = EFAULT;
@@ -2665,23 +2402,32 @@ FileIO_PrivilegedPosixOpen(const char *pathName,  // IN:
 
    ASSERT((flags & (O_CREAT | O_TRUNC)) == 0);
 
-   if (Id_IsSuperUser()) {
-      suDone = FALSE;
-   } else {
-      uid = Id_BeginSuperUser();
-      suDone = TRUE;
+#if defined(__APPLE__)
+   if (privilegedOpenerFunc != NULL) {
+      fd = (*privilegedOpenerFunc)(pathName, flags);
+   } else
+#endif
+   {
+      Bool suDone;
+      uid_t uid = -1;
+
+      if (Id_IsSuperUser()) {
+         suDone = FALSE;
+      } else {
+         uid = Id_BeginSuperUser();
+         suDone = TRUE;
+      }
+
+      fd = Posix_Open(pathName, flags, 0);
+
+      if (suDone) {
+         int error = errno;
+
+         Id_EndSuperUser(uid);
+         errno = error;
+      }
    }
 
-   fd = Posix_Open(pathName, flags, 0);
-
-   if (suDone) {
-      int error = errno;
-
-      Id_EndSuperUser(uid);
-      errno = error;
-   }
-
-
    return fd;
 }
 
@@ -2981,15 +2727,15 @@ void
 FileIOAligned_PoolExit(void)
 {
    if (!alignedPool.lock) {
-      LOG_ONCE(("%s called without FileIOAligned_Pool lock\n", __FUNCTION__));
+      LOG_ONCE("%s called without FileIOAligned_Pool lock\n", __FUNCTION__);
       return;
    }
 
    MXUser_AcquireExclLock(alignedPool.lock);
 
    if (alignedPool.numBusy > 0) {
-      LOG_ONCE(("%s: %d busy buffers!  Proceeding with trepidation.\n",
-		__FUNCTION__, alignedPool.numBusy));
+      LOG_ONCE("%s: %d busy buffers!  Proceeding with trepidation.\n",
+               __FUNCTION__, alignedPool.numBusy);
    }
    while (alignedPool.numAlloc > 0) {
       alignedPool.numAlloc--;
@@ -3029,7 +2775,7 @@ FileIOAligned_PoolMalloc(size_t size)  // IN:
    void *buf = NULL;
 
    if (!alignedPool.lock) {
-      LOG_ONCE(("%s called without FileIOAligned_Pool lock\n", __FUNCTION__));
+      LOG_ONCE("%s called without FileIOAligned_Pool lock\n", __FUNCTION__);
       return NULL;
    }
 
@@ -3091,7 +2837,7 @@ FileIOAligned_PoolFree(void *ptr)  // IN:
    VmTimeType now;
 
    if (!alignedPool.lock) {
-      LOG_ONCE(("%s called without FileIOAligned_Pool lock\n", __FUNCTION__));
+      LOG_ONCE("%s called without FileIOAligned_Pool lock\n", __FUNCTION__);
 
       return FALSE;
    }
