@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -53,8 +53,13 @@
 #   include <pwd.h>
 #endif
 
+#if defined(__APPLE__)
+#include <TargetConditionals.h>
+#endif
+
 #include "vmware.h"
 #include "posix.h"
+#include "codeset.h"
 #include "file.h"
 #include "fileInt.h"
 #include "msg.h"
@@ -63,7 +68,6 @@
 #include "util.h"
 #include "timeutil.h"
 #include "dynbuf.h"
-#include "localconfig.h"
 #include "hostType.h"
 #include "vmfs.h"
 #include "hashTable.h"
@@ -96,9 +100,9 @@ static char *FilePosixNearestExistingAncestor(char const *path);
 #endif
 
 struct WalkDirContextImpl {
-   int     cnt;
-   int     iter;
-   char  **files;
+   char       *dirName;
+   DIR        *dir;
+   HashTable  *hash;
 };
 
 
@@ -107,6 +111,8 @@ struct WalkDirContextImpl {
 #define FS_NFS_ON_ESX "NFS"
 /* A string for VMFS on ESX file system type */
 #define FS_VMFS_ON_ESX "VMFS"
+/* A string for vsanD on ESX file system type */
+#define FS_VSAND_ON_ESX "vsanD"
 #define FS_VSAN_URI_PREFIX      "vsan:"
 
 #if defined __ANDROID__
@@ -574,11 +580,12 @@ File_StripFwdSlashes(const char *pathName)  // IN:
  *      path of "." is the current working directory, ".." is parent
  *      directory and so on. If the path is NULL or "", this routine
  *      returns the current working directory.
- *      There are mainly two use cases for this function. To find the
- *      canonical path of a file or directory that exists, or when we
- *      are about to create a child in an existing parent directory.
- *      For other uses cases, this routine is not guaranteed to return
- *      a canonical path.
+ *
+ *      On FreeBSD and Sun platforms, this routine will only work if
+ *      the path exists, or when we are about to create a child in an
+ *      existing parent directory. This is because on these platforms,
+ *      we cannot rely on finding existing ancestor and such because
+ *      those functions are not compiled.
  *
  * Results:
  *      NULL if error (reported to the user)
@@ -619,18 +626,41 @@ File_FullPath(const char *pathName)  // IN:
       if (ret == NULL) {
          char *dir;
          char *file;
+#if defined(__FreeBSD__) || defined(sun)
          char *realDir;
+#else
+         char *ancestorPath;
+         char *ancestorRealPath;
+#endif
 
          File_GetPathName(path, &dir, &file);
+#if defined(__FreeBSD__) || defined(sun)
          realDir = Posix_RealPath(dir);
          if (realDir == NULL) {
             realDir = File_StripFwdSlashes(dir);
          }
 
          ret = Unicode_Join(realDir, DIRSEPS, file, NULL);
+         Posix_Free(realDir);
+#else
+         ancestorPath = FilePosixNearestExistingAncestor(dir);
+         ancestorRealPath = Posix_RealPath(ancestorPath);
+
+         /*
+          * Check if the ancestor was deleted before we could compute its
+          * realPath
+          */
+         if (ancestorRealPath == NULL) {
+            ret = File_StripFwdSlashes(path);
+         } else {
+            ret = File_PathJoin(ancestorRealPath, path + strlen(ancestorPath));
+            Posix_Free(ancestorRealPath);
+         }
+
+         Posix_Free(ancestorPath);
+#endif
          Posix_Free(dir);
          Posix_Free(file);
-         Posix_Free(realDir);
       }
 
       Posix_Free(path);
@@ -964,7 +994,7 @@ File_SetFilePermissions(const char *pathName,  // IN:
  *-----------------------------------------------------------------------------
  */
 
-static Bool
+Bool
 FilePosixGetParent(char **canPath)  // IN/OUT: Canonical file path
 {
    char *pathName;
@@ -1021,6 +1051,7 @@ File_GetParent(char **canPath)  // IN/OUT: Canonical file path
 }
 
 
+#if !defined(__APPLE__) || TARGET_OS_IPHONE
 /*
  *----------------------------------------------------------------------
  *
@@ -1040,7 +1071,7 @@ File_GetParent(char **canPath)  // IN/OUT: Canonical file path
  *----------------------------------------------------------------------
  */
 
-static Bool
+Bool
 FileGetStats(const char *pathName,       // IN:
              Bool doNotAscend,           // IN:
              struct statfs *pstatfsbuf)  // OUT:
@@ -1048,8 +1079,7 @@ FileGetStats(const char *pathName,       // IN:
    Bool retval = TRUE;
    char *dupPath = NULL;
 
-   while (Posix_Statfs(dupPath ? dupPath : pathName,
-                             pstatfsbuf) == -1) {
+   while (Posix_Statfs(dupPath ? dupPath : pathName, pstatfsbuf) == -1) {
       if (errno != ENOENT || doNotAscend) {
          retval = FALSE;
          break;
@@ -1102,7 +1132,7 @@ File_GetFreeSpace(const char *pathName,  // IN: File name
    }
 
    if (FileGetStats(fullPath, doNotAscend, &statfsbuf)) {
-      ret = (uint64) statfsbuf.f_bavail * statfsbuf.f_bsize;
+      ret = (uint64) statfsbuf.f_bavail * statfsbuf.f_bsize;  // available space
    } else {
       Warning("%s: Couldn't statfs %s\n", __func__, fullPath);
       ret = -1;
@@ -1112,6 +1142,7 @@ File_GetFreeSpace(const char *pathName,  // IN: File name
 
    return ret;
 }
+#endif
 
 
 #if defined(VMX86_SERVER)
@@ -1459,6 +1490,10 @@ FileIsVMFS(const char *pathName)  // IN:
       /* We want to match anything that starts with VMFS */
       result = strncmp(fsAttrs->fsType, FS_VMFS_ON_ESX,
                        strlen(FS_VMFS_ON_ESX)) == 0;
+      if (!result) {
+         result = strncmp(fsAttrs->fsType, FS_VSAND_ON_ESX,
+                          strlen(FS_VSAND_ON_ESX)) == 0;
+      }
    } else {
       Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
    }
@@ -1574,13 +1609,16 @@ File_SupportsOptimisticLock(const char *pathName)  // IN:
 }
 
 
+#if !defined(__APPLE__) || TARGET_OS_IPHONE
 /*
  *----------------------------------------------------------------------
  *
  * File_GetCapacity --
  *
- *      Return the total capacity (in bytes) available to the user on a disk
- *      where a file is or would be
+ *      Return the total capacity (in bytes) of the file system that the
+ *      specified file resides on. This is not be confused with the
+ *      amount of free space available in the file system the specified
+ *      file resides on.
  *
  * Results:
  *      -1 if error (reported to the user)
@@ -1604,7 +1642,7 @@ File_GetCapacity(const char *pathName)  // IN: Path name
    }
 
    if (FileGetStats(fullPath, FALSE, &statfsbuf)) {
-      ret = (uint64) statfsbuf.f_blocks * statfsbuf.f_bsize;
+      ret = (uint64) statfsbuf.f_blocks * statfsbuf.f_bsize; // FS size
    } else {
       Warning(LGPFX" %s: Couldn't statfs\n", __func__);
       ret = -1;
@@ -1614,6 +1652,7 @@ File_GetCapacity(const char *pathName)  // IN: Path name
 
    return ret;
 }
+#endif
 
 
 /*
@@ -1952,6 +1991,13 @@ retry:
                   Str_Strcpy(canPath, ptr, sizeof canPath);
                }
             } else {
+
+               /*
+                * This is coded as it is in order to document clearly that
+                * the function does not handle bind mount correctly (it
+                * always assumes rbind).
+                */
+               /* coverity[dead_error_line] */
                Str_Strcpy(canPath, ptr, sizeof canPath);
             }
 
@@ -2501,9 +2547,9 @@ FileVMKGetMaxOrSupportsFileSize(const char *pathName,  // IN:
    /*
     * Try the old way if IOCTL failed.
     */
-   LOG(0, (LGPFX" %s: Failed to figure out max file size via "
-           "IOCTLCMD_VMFS_GET_MAX_FILE_SIZE. Falling back to old method.\n",
-           __func__));
+   LOG(0, LGPFX" %s: Failed to figure out max file size via "
+       "IOCTLCMD_VMFS_GET_MAX_FILE_SIZE. Falling back to old method.\n",
+       __func__);
    maxFileSize = -1;
 
    if (File_GetVMFSAttributes(pathName, &fsAttrs) < 0) {
@@ -2812,126 +2858,64 @@ FileCreateDirectory(const char *pathName,  // IN:
  *----------------------------------------------------------------------
  */
 
-static int
-FileKeyDispose(const char *key,   // IN:
-               void *value,       // IN:
-               void *clientData)  // IN:
-{
-   Posix_Free((void *) key);
-
-   return 0;
-}
+typedef struct {
+   char   **fileNames;
+   uint32   pos;
+} ListParams;
 
 static int
-FileUnique(const char *key,   // IN:
-           void *value,       // IN:
-           void *clientData)  // IN/OUT: a DynBuf
+FileNameArray(const char *key,   // IN:
+              void *value,       // IN:
+              void *clientData)  // IN/OUT: ListParams pointer
 {
-   DynBuf *b = clientData;
+   ListParams *params = clientData;
 
-   DynBuf_Append(b, &key, sizeof key);
+   ASSERT(value == NULL);
+
+   params->fileNames[params->pos++] = Util_SafeStrdup(key);
 
    return 0;
 }
 
 int
-File_ListDirectory(const char *pathName,  // IN:
-                   char ***ids)           // OUT: relative paths
+File_ListDirectory(const char *dirName,  // IN:
+                   char ***ids)          // OUT: relative paths
 {
-   int err;
-   DIR *dir;
-   int count;
-   HashTable *hash;
+   int count = -1;
+   WalkDirContext context = File_WalkDirectoryStart(dirName);
 
-   ASSERT(pathName != NULL);
+   if (context != NULL) {
+      int err;
 
-   dir = Posix_OpenDir(pathName);
+      while (File_WalkDirectoryNext(context, NULL))
+         ;
 
-   if (dir == NULL) {
-      // errno is preserved
-      return -1;
-   }
-
-   hash = HashTable_Alloc(256, HASH_STRING_KEY, NULL);
-   count = 0;
-
-   while (TRUE) {
-      struct dirent *entry;
-
-      errno = 0;
-      entry = readdir(dir);
-      if (entry == NULL) {
-         err = errno;
-         break;
-      }
-
-      /* Strip out undesirable paths.  No one ever cares about these. */
-      if ((strcmp(entry->d_name, ".") == 0) ||
-          (strcmp(entry->d_name, "..") == 0)) {
-         continue;
-      }
-
-      /* Don't create the file list if we aren't providing it to the caller. */
-      if (ids) {
-         char *id;
-
-         if (Unicode_IsBufferValid(entry->d_name, -1,
-                                   STRING_ENCODING_DEFAULT)) {
-            id = Unicode_Alloc(entry->d_name, STRING_ENCODING_DEFAULT);
-         } else {
-            id = Unicode_EscapeBuffer(entry->d_name, -1,
-                                     STRING_ENCODING_DEFAULT);
-
-            Warning("%s: file '%s' in directory '%s' cannot be converted to "
-                    "UTF8\n", __FUNCTION__, pathName, id);
-
-            Posix_Free(id);
-
-            id = Unicode_Duplicate(UNICODE_SUBSTITUTION_CHAR
-                                   UNICODE_SUBSTITUTION_CHAR
-                                   UNICODE_SUBSTITUTION_CHAR);
-         }
-
-         /*
-          * It is possible for a directory operation to change the contents
-          * of a directory while this routine is running. If the OS decides
-          * to physically rearrange the contents of the directory it is
-          * possible for readdir to report a file more than once. Only add
-          * a file to the return data if it is unique within the return
-          * data.
-          */
-
-         if (HashTable_Insert(hash, id, NULL)) {
-            count++;
-         } else {
-            Posix_Free(id);
-         }
-      } else {
-         count++;
-      }
-   }
-
-   closedir(dir);
-
-   if (ids) {
-      ASSERT(count == HashTable_GetNumElements(hash));
+      err = errno;
 
       if (err == 0) {
-         DynBuf b;
+         count = HashTable_GetNumElements(context->hash);
 
-         DynBuf_Init(&b);
+         if (ids) {
+            if (count == 0) {
+               *ids = NULL;
+            } else {
+               ListParams params;
 
-         HashTable_ForEach(hash, FileUnique, &b);
-         *ids = DynBuf_Detach(&b);
-         DynBuf_Destroy(&b);
-      } else {
-         HashTable_ForEach(hash, FileKeyDispose, NULL);
+               params.fileNames = Util_SafeCalloc(count, sizeof(char *));
+               params.pos = 0;
+
+               HashTable_ForEach(context->hash, FileNameArray, &params);
+               *ids = params.fileNames;
+            }
+         }
       }
+
+      File_WalkDirectoryEnd(context);
+
+      errno = err;
    }
 
-   HashTable_Free(hash);
-
-   return (errno = err) == 0 ? count : -1;
+   return count;
 }
 
 
@@ -2951,13 +2935,29 @@ File_ListDirectory(const char *pathName,  // IN:
  *-----------------------------------------------------------------------------
  */
 
+static int
+FileKeyDispose(const char *key,   // IN:
+               void *value,       // IN:
+               void *clientData)  // IN:
+{
+   Posix_Free((void *) key);
+
+   return 0;
+}
+
 void
 File_WalkDirectoryEnd(WalkDirContext context)  // IN:
 {
    if (context != NULL) {
-      if (context->cnt > 0) {
-         Util_FreeStringList(context->files, context->cnt);
+      if (context->dir != NULL) {
+         closedir(context->dir);
       }
+
+      HashTable_ForEach(context->hash, FileKeyDispose, NULL);
+
+      HashTable_Free(context->hash);
+
+      Posix_Free(context->dirName);
       Posix_Free(context);
    }
 }
@@ -2987,19 +2987,25 @@ File_WalkDirectoryEnd(WalkDirContext context)  // IN:
  */
 
 WalkDirContext
-File_WalkDirectoryStart(const char *parentPath)  // IN:
+File_WalkDirectoryStart(const char *dirName)  // IN:
 {
-   WalkDirContextImpl *context = malloc(sizeof *context);
+   WalkDirContextImpl *context;
 
-   if (context != NULL) {
-      context->files = NULL;
-      context->iter = 0;
-      context->cnt = File_ListDirectory(parentPath, &context->files);
+   ASSERT(dirName != NULL);
 
-      if (context->cnt == -1) {
-         File_WalkDirectoryEnd(context);
-         context = NULL;
-      }
+   context = Util_SafeMalloc(sizeof *context);
+
+   context->dirName = Util_SafeStrdup(dirName);
+   context->hash = HashTable_Alloc(256, HASH_STRING_KEY, NULL);
+   context->dir = Posix_OpenDir(dirName);
+
+   if (context->dir == NULL) {
+      int err = errno;
+
+      File_WalkDirectoryEnd(context);
+
+      errno = err;
+      context = NULL;
    }
 
    return context;
@@ -3011,16 +3017,15 @@ File_WalkDirectoryStart(const char *parentPath)  // IN:
  *
  * File_WalkDirectoryNext --
  *
- *      Get the next entry in a directory traversal started with
+ *      Get the next file name during a directory traversal started with
  *      File_WalkDirectoryStart.
  *
  * Results:
- *      TRUE iff the traversal hasn't completed.
- *
- *      If TRUE, *path holds an allocated string of a directory entry that the
- *      caller must free (see free).
- *
- *      If FALSE, errno is 0 iff the walk completed sucessfully.
+ *      TRUE   Traversal hasn't completed yet. If *fileName is not NULL,
+ *             an allocated string of the file name found is returned.
+ *             The caller must free the string.
+ *      FALSE  The traversal has completed. Check errno; if it is zero (0),
+ *             the walk completed sucessfully.
  *
  * Side effects:
  *      None
@@ -3030,19 +3035,76 @@ File_WalkDirectoryStart(const char *parentPath)  // IN:
 
 Bool
 File_WalkDirectoryNext(WalkDirContext context,  // IN:
-                       char **path)             // OUT:
+                       char **fileName)         // OUT/OPT:
 {
-   ASSERT(context);
-   ASSERT(path != NULL);
+   int err = 0;
+   Bool callAgain = FALSE;
 
-   errno = 0;  // Any errors showed up at "start time".
+   ASSERT(context != NULL);
 
-   if (context->iter < context->cnt) {
-      *path = Util_SafeStrdup(context->files[context->iter++]);
-      return TRUE;
+   while (TRUE) {
+      char *allocName;
+      struct dirent *entry;
+
+      errno = 0;
+      entry = readdir(context->dir);
+      if (entry == NULL) {
+         err = errno;
+         break;
+      }
+
+      /* Strip out undesirable paths. No one ever cares about these. */
+      if ((strcmp(entry->d_name, ".") == 0) ||
+          (strcmp(entry->d_name, "..") == 0)) {
+         continue;
+      }
+
+      /*
+       * It is possible for a directory operation to change the contents
+       * of a directory while this routine is running. If the OS decides
+       * to physically rearrange the contents of the directory it is
+       * possible for readdir to report a file more than once. Only add
+       * a file to the return data if it is unique within the return
+       * data.
+       */
+
+#if defined(VMX86_SERVER) || defined(__APPLE__)  // UTF8 native platforms
+      if (CodeSet_IsStringValidUTF8(entry->d_name)) {
+         allocName = Util_SafeStrdup(entry->d_name);
+#else
+      if (Unicode_IsBufferValid(entry->d_name, -1, STRING_ENCODING_DEFAULT)) {
+         allocName = Unicode_Alloc(entry->d_name, STRING_ENCODING_DEFAULT);
+#endif
+      } else {
+         char *id = Unicode_EscapeBuffer(entry->d_name, -1,
+                                         STRING_ENCODING_DEFAULT);
+
+         Warning("%s: file '%s' in directory '%s' cannot be converted to "
+                 "UTF8\n", __FUNCTION__, context->dirName, id);
+
+         Posix_Free(id);
+
+         allocName = Unicode_Duplicate(UNICODE_SUBSTITUTION_CHAR
+                                       UNICODE_SUBSTITUTION_CHAR
+                                       UNICODE_SUBSTITUTION_CHAR);
+      }
+
+      if (HashTable_Insert(context->hash, allocName, NULL)) {  // Unique - good
+         if (fileName != NULL) {
+            *fileName = Util_SafeStrdup(allocName);
+         }
+
+         callAgain = TRUE;
+         break;
+      } else {  // Duplicate - ignore
+         free(allocName);
+         continue;
+      }
    }
 
-   return FALSE;
+   errno = err;
+
+   return callAgain;
 }
 
 
