@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -21,6 +21,7 @@
 
 #define GOBUILD_TOOLS_VERSION_USED
 #include "vmtools_version.h"
+#undef GOBUILD_TOOLS_VERSION_USED
 
 #else
 
@@ -46,6 +47,7 @@
  *          reformating or changing macro names is being performed
  *          please run the perl script to ensure it succeeds.
  */
+/* clang-format off */
 
 
 /*
@@ -1453,9 +1455,234 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_CHAINSAW_RELEASE_V_MNR  2
 #define   TOOLS_VERSION_CHAINSAW_RELEASE_V_BASE 0
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_CHAINSAW_RELEASE
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_CHAINSAW_RELEASE)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_CHAINSAW_RELEASE)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CHAINSAW_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CHAINSAW_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_MJR  10
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_MNR  2
+#define   TOOLS_VERSION_CHAINSAW_PATCH2_V_BASE 7
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH1_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH1_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_APPDEF_BETA TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_APPDEF_BETA_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_APPDEF_BETA_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_APPDEF_BETA_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_APPDEF_BETA_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH2_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH2_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH3_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH3_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH3_V_BASE 7
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE2_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE2_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE2_V_BASE 10
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH4 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH4_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH4_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH4_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH4_V_BASE 11
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH5 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH5_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH5_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH5_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH5_V_BASE 13
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH6 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH6_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH6_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH6_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH6_V_BASE 14
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH7 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH7_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH7_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH7_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH7_V_BASE 15
+
+#ifndef RC_INVOKED
+
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE3_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE3_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_UPDATE3_V_BASE 20
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH8 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH8_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH8_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH8_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH8_V_BASE 21
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH9 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH9_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH9_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH9_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH9_V_BASE 22
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH10 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH10_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH10_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH10_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH10_V_BASE 23
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_JACKHAMMER_PATCH11 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_JACKHAMMER_PATCH11_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_JACKHAMMER_PATCH11_V_MJR  10
+#define   TOOLS_VERSION_JACKHAMMER_PATCH11_V_MNR  3
+#define   TOOLS_VERSION_JACKHAMMER_PATCH11_V_BASE 24
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SLEDGEHAMMER_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SLEDGEHAMMER_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SLEDGEHAMMER_RELEASE_V_MJR  11
+#define   TOOLS_VERSION_SLEDGEHAMMER_RELEASE_V_MNR  0
+#define   TOOLS_VERSION_SLEDGEHAMMER_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SLEDGEHAMMER_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH1_V_MJR  11
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH1_V_MNR  0
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SLEDGEHAMMER_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SLEDGEHAMMER_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SLEDGEHAMMER_UPDATE1_V_MJR  11
+#define   TOOLS_VERSION_SLEDGEHAMMER_UPDATE1_V_MNR  0
+#define   TOOLS_VERSION_SLEDGEHAMMER_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_SLEDGEHAMMER_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH2_V_MJR  11
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH2_V_MNR  0
+#define   TOOLS_VERSION_SLEDGEHAMMER_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_BALLPEENHAMMER_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_BALLPEENHAMMER_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_BALLPEENHAMMER_RELEASE_V_MJR  11
+#define   TOOLS_VERSION_BALLPEENHAMMER_RELEASE_V_MNR  1
+#define   TOOLS_VERSION_BALLPEENHAMMER_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_BALLPEENHAMMER_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH1_V_MJR  11
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH1_V_MNR  1
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_BALLPEENHAMMER_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_BALLPEENHAMMER_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_BALLPEENHAMMER_UPDATE1_V_MJR  11
+#define   TOOLS_VERSION_BALLPEENHAMMER_UPDATE1_V_MNR  1
+#define   TOOLS_VERSION_BALLPEENHAMMER_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_BALLPEENHAMMER_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH2_V_MJR  11
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH2_V_MNR  1
+#define   TOOLS_VERSION_BALLPEENHAMMER_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CRESCENT_WRENCH_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CRESCENT_WRENCH_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CRESCENT_WRENCH_RELEASE_V_MJR  11
+#define   TOOLS_VERSION_CRESCENT_WRENCH_RELEASE_V_MNR  2
+#define   TOOLS_VERSION_CRESCENT_WRENCH_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CRESCENT_WRENCH_PATCH1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH1_V_MJR  11
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH1_V_MNR  2
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CRESCENT_WRENCH_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CRESCENT_WRENCH_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CRESCENT_WRENCH_UPDATE1_V_MJR  11
+#define   TOOLS_VERSION_CRESCENT_WRENCH_UPDATE1_V_MNR  2
+#define   TOOLS_VERSION_CRESCENT_WRENCH_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_CRESCENT_WRENCH_PATCH2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH2_V_MJR  11
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH2_V_MNR  2
+#define   TOOLS_VERSION_CRESCENT_WRENCH_PATCH2_V_BASE 6
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_NEXT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_NEXT_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_NEXT_V_MJR  11
+#define   TOOLS_VERSION_NEXT_V_MNR  3
+#define   TOOLS_VERSION_NEXT_V_BASE 0
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_NEXT
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_NEXT)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_NEXT)
 
 /*
  * The extended Tools version is the current Tools version with the
@@ -1472,6 +1699,8 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define TOOLS_VERSION_EXT_CURRENT_STR \
    TOOLS_VERSION_CURRENT_STR "." PRODUCT_BUILD_NUMBER_NUMERIC_STRING
 
+/* clang-format on */
+
 #endif /* VM_TOOLS_VERSION_H */
 
 #endif /* defined(USE_GOBUILD_TOOLS_VERSION) && !defined(GOBUILD_TOOLS_VERSION_USED) */
