@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2014,2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2014,2017-2020,2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -95,6 +95,7 @@ typedef enum {
    ETH_HEADER_TYPE_802_1PQ,
    ETH_HEADER_TYPE_802_3,
    ETH_HEADER_TYPE_802_1PQ_802_3,
+   ETH_HEADER_TYPE_NESTED_802_1PQ,
 } Eth_HdrType;
 
 // DIX type fields we care about
@@ -107,7 +108,11 @@ typedef enum {
    ETH_TYPE_CDP         = 0x2000,
    ETH_TYPE_AKIMBI      = 0x88DE,
    ETH_TYPE_VMWARE      = 0x8922,
+   ETH_TYPE_1588        = 0x88F7,
+   ETH_TYPE_NSH         = 0x894F,
    ETH_TYPE_802_1PQ     = 0x8100,  // not really a DIX type, but used as such
+   ETH_TYPE_QINQ        = 0x88A8,
+   ETH_TYPE_PAE         = 0x888e,  /* EAPOL PAE/802.1x */
    ETH_TYPE_LLC         = 0xFFFF,  // 0xFFFF is IANA reserved, used to mark LLC
 } Eth_DixType;
 typedef enum {
@@ -119,7 +124,11 @@ typedef enum {
    ETH_TYPE_CDP_NBO     = 0x0020,
    ETH_TYPE_AKIMBI_NBO  = 0xDE88,
    ETH_TYPE_VMWARE_NBO  = 0x2289,
+   ETH_TYPE_1588_NBO    = 0xF788,
+   ETH_TYPE_NSH_NBO     = 0x4F89,
    ETH_TYPE_802_1PQ_NBO = 0x0081,  // not really a DIX type, but used as such
+   ETH_TYPE_QINQ_NBO    = 0xA888,
+   ETH_TYPE_PAE_NBO     = 0x8e88,   /* EAPOL PAE/802.1x */
    ETH_TYPE_802_3_PAUSE_NBO = 0x0888,  // pause frame based ethernet flow control
 } Eth_DixTypeNBO;
 
@@ -132,13 +141,11 @@ typedef enum {
 
 #define ETH_LLC_CONTROL_UFRAME_MASK (0x3)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_DIX {
+#pragma pack(push, 1)
+typedef struct Eth_DIX {
    uint16  typeNBO;     // indicates the higher level protocol
-}
-#include "vmware_pack_end.h"
-Eth_DIX;
+} Eth_DIX;
+#pragma pack(pop)
 
 /*
  * LLC header come in two varieties:  8 bit control and 16 bit control.
@@ -146,44 +153,36 @@ Eth_DIX;
  * indicated the 8 bit control field.
  */
 
-typedef 
-#include "vmware_pack_begin.h"
-struct Eth_LLC8 {
+#pragma pack(push, 1)
+typedef struct Eth_LLC8 {
    uint8   dsap;
    uint8   ssap;
    uint8   control;
-}
-#include "vmware_pack_end.h"
-Eth_LLC8;
+} Eth_LLC8;
+#pragma pack(pop)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_LLC16 {
+#pragma pack(push, 1)
+typedef struct Eth_LLC16 {
    uint8   dsap;
    uint8   ssap;
    uint16  control;
-}
-#include "vmware_pack_end.h"
-Eth_LLC16;
+} Eth_LLC16;
+#pragma pack(pop)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_SNAP {
+#pragma pack(push, 1)
+typedef struct Eth_SNAP {
    uint8   snapOrg[3];
    Eth_DIX snapType;
-} 
-#include "vmware_pack_end.h"
-Eth_SNAP;
+} Eth_SNAP;
+#pragma pack(pop)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_802_3 {  
+#pragma pack(push, 1)
+typedef struct Eth_802_3 {  
    uint16   lenNBO;      // length of the frame
    Eth_LLC8 llc;         // LLC header
    Eth_SNAP snap;        // SNAP header
-} 
-#include "vmware_pack_end.h"
-Eth_802_3;
+} Eth_802_3;
+#pragma pack(pop)
 
 // 802.1p QOS/priority tags
 // 
@@ -198,33 +197,32 @@ enum {
    ETH_802_1_P_NETWORK_CONTROL      = 7
 };
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_802_1pq_Tag {
+#pragma pack(push, 1)
+typedef struct Eth_802_1pq_Tag {
    uint16 typeNBO;            // always ETH_TYPE_802_1PQ
    uint16 vidHi:4,            // 802.1q vlan ID high nibble
           canonical:1,        // bit order? (should always be 0)
           priority:3,         // 802.1p priority tag
           vidLo:8;            // 802.1q vlan ID low byte
-}
-#include "vmware_pack_end.h"
-Eth_802_1pq_Tag;
+} Eth_802_1pq_Tag;
+#pragma pack(pop)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_802_1pq {
+#pragma pack(push, 1)
+typedef struct Eth_802_1pq {
    Eth_802_1pq_Tag tag;       // VLAN/QOS tag
    union {
       Eth_DIX      dix;       // DIX header follows
       Eth_802_3    e802_3;    // or 802.3 header follows 
+      struct {
+         Eth_802_1pq_Tag   tag;        // inner VLAN/QOS tag
+         Eth_DIX           dix;        // DIX header follows
+      } nested802_1pq;
    }; 
-}
-#include "vmware_pack_end.h"
-Eth_802_1pq; 
+} Eth_802_1pq; 
+#pragma pack(pop)
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_Header {
+#pragma pack(push, 1)
+typedef struct Eth_Header {
    Eth_Address     dst;       // all types of ethernet frame have dst first
    Eth_Address     src;       // and the src next (at least all the ones we'll see)
    union {
@@ -232,9 +230,8 @@ struct Eth_Header {
       Eth_802_3    e802_3;    // ...or an 802.3 header
       Eth_802_1pq  e802_1pq;  // ...or an 802.1[pq] tag and a header
    };
-}
-#include "vmware_pack_end.h"
-Eth_Header;
+} Eth_Header;
+#pragma pack(pop)
 
 /*
  * Official VMware ethertype header format and types
@@ -249,15 +246,13 @@ enum {
    ETH_VMWARE_FRAME_TYPE_LLC        = 4, // XXX: Just re-use COLOR?
 };
 
-typedef
-#include "vmware_pack_begin.h"
-struct Eth_VMWareFrameHeader {
+#pragma pack(push, 1)
+typedef struct Eth_VMWareFrameHeader {
    uint32         magic;
    uint16         lenNBO;
    uint8          type;
-}
-#include "vmware_pack_end.h"
-Eth_VMWareFrameHeader;
+} Eth_VMWareFrameHeader;
+#pragma pack(pop)
 
 typedef Eth_Header Eth_802_1pq_Header; // for sizeof
 
@@ -312,6 +307,11 @@ extern Eth_Address netEthBroadcastAddr;
                                       sizeof(Eth_Address) + \
                                       sizeof(Eth_802_1pq_Tag) + \
                                       sizeof(Eth_802_3))
+#define ETH_HEADER_LEN_NESTED_802_1PQ  (sizeof(Eth_Address) + \
+                                        sizeof(Eth_Address) + \
+                                        sizeof(Eth_802_1pq_Tag) + \
+                                        sizeof(Eth_802_1pq_Tag) + \
+                                        sizeof(Eth_DIX))
 
 #define ETH_MIN_HEADER_LEN   (ETH_HEADER_LEN_DIX)
 #define ETH_MAX_HEADER_LEN   (ETH_HEADER_LEN_802_1PQ_802_3)
@@ -319,7 +319,13 @@ extern Eth_Address netEthBroadcastAddr;
 #define ETH_MIN_FRAME_LEN                    60
 #define ETH_MAX_STD_MTU                      1500
 #define ETH_MAX_STD_FRAMELEN                 (ETH_MAX_STD_MTU + ETH_MAX_HEADER_LEN)
-#define ETH_MAX_JUMBO_MTU                    9000
+/*
+ * ENS_MBUF_SLAB_9K_ALLOC_SIZE and PKT_SLAB_JUMBO_SIZE both use 9216 for L2
+ * MTU. And ETH_MAX_JUMBO_MTU is L3 MTU. It is required to have
+ * (ETH_MAX_JUMBO_MTU + ETH_MAX_HEADER_LEN) <= 9216 and ETH_MAX_HEADER_LEN is
+ * 26. So the maximal possible ETH_MAX_JUMBO_MTU = 9216 - 26 = 9190.
+ */
+#define ETH_MAX_JUMBO_MTU                    9190
 #define ETH_MAX_JUMBO_FRAMELEN               (ETH_MAX_JUMBO_MTU + ETH_MAX_HEADER_LEN)
 
 #define ETH_DEFAULT_MTU                      1500
@@ -440,6 +446,7 @@ Eth_IsNullAddr(const Eth_Address addr)
  *
  *      ETH_HEADER_TYPE_DIX: typical 14 byte eth header
  *      ETH_HEADER_TYPE_802_1PQ: DIX+vlan tagging
+ *      ETH_HEADER_TYPE_NESTED_802_1PQ: DIX+vlan tagging+DIX+vlan tagging
  *      ETH_HEADER_TYPE_802_3: 802.3 eth header
  *      ETH_HEADER_TYPE_802_1PQ_802_3: 802.3 + vlan tag 
  *
@@ -492,6 +499,10 @@ Eth_HeaderType(const Eth_Header *eh)
           * vlan tagging with dix style type
           */
 
+         if (UNLIKELY(eh->e802_1pq.dix.typeNBO == ETH_TYPE_802_1PQ_NBO)) {
+            return ETH_HEADER_TYPE_NESTED_802_1PQ;
+         }
+
          return ETH_HEADER_TYPE_802_1PQ;
       }
 
@@ -537,37 +548,39 @@ Eth_EncapsulatedPktType(const Eth_Header *eh)
    Eth_HdrType type = Eth_HeaderType(eh);  
 
    switch (type) {
-      
-      case ETH_HEADER_TYPE_DIX :
-	 return eh->dix.typeNBO;
+   case ETH_HEADER_TYPE_DIX :
+      return eh->dix.typeNBO;
 
-      case ETH_HEADER_TYPE_802_1PQ :
-	 return eh->e802_1pq.dix.typeNBO;
+   case ETH_HEADER_TYPE_802_1PQ :
+      return eh->e802_1pq.dix.typeNBO;
 
-      case ETH_HEADER_TYPE_802_3 :
-         /*
-          * Documentation describes SNAP headers as having ONLY
-          * 0x03 as the control fields, not just the lower two bits
-          * This prevents the use of Eth_IsLLCControlUFormat.
-          */
-         if ((eh->e802_3.llc.dsap == 0xaa) &&
-              (eh->e802_3.llc.ssap == 0xaa) &&
-              (eh->e802_3.llc.control == ETH_LLC_CONTROL_UFRAME)) {
-                  return eh->e802_3.snap.snapType.typeNBO;
-         } else {
-            // LLC, no snap header, then no type
-            return ETH_TYPE_LLC;
-         }
+   case ETH_HEADER_TYPE_NESTED_802_1PQ:
+      return eh->e802_1pq.nested802_1pq.dix.typeNBO;
 
-      case ETH_HEADER_TYPE_802_1PQ_802_3 :
-         if ((eh->e802_1pq.e802_3.llc.dsap == 0xaa) &&
-              (eh->e802_1pq.e802_3.llc.ssap == 0xaa) &&
-              (eh->e802_1pq.e802_3.llc.control == ETH_LLC_CONTROL_UFRAME)) {
-                  return eh->e802_1pq.e802_3.snap.snapType.typeNBO;
-         } else {
-            // tagged LLC, no snap header, then no type
-            return ETH_TYPE_LLC;
-         }
+   case ETH_HEADER_TYPE_802_3 :
+      /*
+       * Documentation describes SNAP headers as having ONLY
+       * 0x03 as the control fields, not just the lower two bits
+       * This prevents the use of Eth_IsLLCControlUFormat.
+       */
+      if ((eh->e802_3.llc.dsap == 0xaa) &&
+           (eh->e802_3.llc.ssap == 0xaa) &&
+           (eh->e802_3.llc.control == ETH_LLC_CONTROL_UFRAME)) {
+               return eh->e802_3.snap.snapType.typeNBO;
+      } else {
+         // LLC, no snap header, then no type
+         return ETH_TYPE_LLC;
+      }
+
+   case ETH_HEADER_TYPE_802_1PQ_802_3 :
+      if ((eh->e802_1pq.e802_3.llc.dsap == 0xaa) &&
+           (eh->e802_1pq.e802_3.llc.ssap == 0xaa) &&
+           (eh->e802_1pq.e802_3.llc.control == ETH_LLC_CONTROL_UFRAME)) {
+               return eh->e802_1pq.e802_3.snap.snapType.typeNBO;
+      } else {
+         // tagged LLC, no snap header, then no type
+         return ETH_TYPE_LLC;
+      }
    }
 
    ASSERT(FALSE);
@@ -806,7 +819,7 @@ Eth_FillVlanTag(Eth_802_1pq_Tag *tag,
    ASSERT(priority < 8);
 
    tag->typeNBO = ETH_TYPE_802_1PQ_NBO;
-   tag->priority = priority;
+   tag->priority = (uint16)priority;
    tag->canonical = 0;                  // bit order (should be 0)
    tag->vidHi = vlanId >> 8;
    tag->vidLo = vlanId & 0xff;
@@ -911,7 +924,7 @@ Eth_FrameGetPriority(const Eth_Header *eh)
 {
    ASSERT(Eth_IsFrameTagged(eh));
 
-   return eh->e802_1pq.tag.priority;
+   return (uint8)eh->e802_1pq.tag.priority;
 }
 
 
@@ -1081,18 +1094,20 @@ Eth_HeaderLength(const Eth_Header *eh)
    Eth_HdrType type = Eth_HeaderType(eh);  
 
    switch (type) {
+   case ETH_HEADER_TYPE_DIX :
+      return ETH_HEADER_LEN_DIX;
 
-      case ETH_HEADER_TYPE_DIX :
-	 return ETH_HEADER_LEN_DIX;
+   case ETH_HEADER_TYPE_802_1PQ :
+      return ETH_HEADER_LEN_802_1PQ;
 
-      case ETH_HEADER_TYPE_802_1PQ :
-	 return ETH_HEADER_LEN_802_1PQ;
+   case ETH_HEADER_TYPE_NESTED_802_1PQ :
+      return ETH_HEADER_LEN_NESTED_802_1PQ;
 
-      case ETH_HEADER_TYPE_802_3 :
-         return Eth_HeaderLength_802_3(eh);
+   case ETH_HEADER_TYPE_802_3 :
+      return Eth_HeaderLength_802_3(eh);
 
-      case ETH_HEADER_TYPE_802_1PQ_802_3 :
-         return Eth_HeaderLength_802_1PQ_802_3(eh);
+   case ETH_HEADER_TYPE_802_1PQ_802_3 :
+      return Eth_HeaderLength_802_1PQ_802_3(eh);
    }
    
    ASSERT(FALSE);
@@ -1198,17 +1213,25 @@ Eth_IsFrameHeaderComplete(const Eth_Header *eh,
    if (len >= ETH_HEADER_LEN_802_1PQ) {
       /*
        * Eth_HeaderType will correctly enumerate all types once
-       * at least ETH_HEADER_LEN_802_1PQ bytes are available
+       * at least ETH_HEADER_LEN_802_1PQ bytes are available except nested
+       * 802.1pq tag.
        */
       Eth_HdrType type = Eth_HeaderType(eh);
 
-      if (type == ETH_HEADER_TYPE_802_1PQ) {
+      switch (type) {
+      case ETH_HEADER_TYPE_802_1PQ:
          if (ehHdrLen != NULL) {
             *ehHdrLen = ETH_HEADER_LEN_802_1PQ;
          }
          return TRUE;
 
-      } else if (type == ETH_HEADER_TYPE_802_3) {
+      case ETH_HEADER_TYPE_NESTED_802_1PQ:
+         if (ehHdrLen != NULL) {
+            *ehHdrLen = ETH_HEADER_LEN_NESTED_802_1PQ;
+         }
+         return len >= ETH_HEADER_LEN_NESTED_802_1PQ;
+
+      case ETH_HEADER_TYPE_802_3:
          /*
           * Length could be shorter LLC or LLC+SNAP.
           * ETH_HEADER_LEN_802_2_LLC bytes are needed to disambiguate.
@@ -1217,13 +1240,17 @@ Eth_IsFrameHeaderComplete(const Eth_Header *eh,
          ASSERT(ETH_HEADER_LEN_802_1PQ > ETH_HEADER_LEN_802_2_LLC);
          ehLen = Eth_HeaderLength_802_3(eh);
          /* continue to common test */
-      } else if (type == ETH_HEADER_TYPE_802_1PQ_802_3) {
+         break;
+
+      case ETH_HEADER_TYPE_802_1PQ_802_3:
          if (len < ETH_HEADER_LEN_802_1PQ_LLC) {
             return FALSE;
          }
          ehLen = Eth_HeaderLength_802_1PQ_802_3(eh);
          /* continue to common test */
-      } else {
+         break;
+
+      default:
          /*
           * This else clause is unreachable, but if removed
           * compiler complains about ehLen possibly being
