@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2017 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2021 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -275,7 +275,15 @@ TimeSyncReadHost(int64 *host, int64 *apparentError, Bool *apparentErrorValid,
                  "attempting BDOOR_CMD_GETTIME\n");
          bp.in.cx.halfs.low = BDOOR_CMD_GETTIME;
          Backdoor(&bp);
-         hostSecs = bp.out.ax.word;
+         /*
+          * This backdoor returns uint32 time value in bp.out.ax.word or
+          * MAX_UINT32 in case of error.
+          */
+         if (bp.out.ax.word == MAX_UINT32) {
+            hostSecs = -1;
+         } else {
+            hostSecs = bp.out.ax.word;
+         }
       }
    }
    hostUsecs = bp.out.bx.word;
@@ -459,9 +467,9 @@ TimeSyncSlewTime(TimeSyncData *data, int64 adjustment)
 
    int64 now;
    int64 remaining = 0;
-   int64 timeSyncPeriodUS = data->timeSyncPeriod * US_PER_SEC;
+   int64 timeSyncPeriodUS = (int64)data->timeSyncPeriod * US_PER_SEC;
    int64 slewDiff = (adjustment * data->slewPercentCorrection) / 100;
-   
+
    if (!TimeSync_GetCurrentTime(&now)) {
       return FALSE;
    }
@@ -537,7 +545,7 @@ static void
 TimeSyncResetSlew(TimeSyncData *data)
 {
    int64 remaining;
-   int64 timeSyncPeriodUS = data->timeSyncPeriod * US_PER_SEC;
+   int64 timeSyncPeriodUS = (int64)data->timeSyncPeriod * US_PER_SEC;
    data->slewState = TimeSyncUncalibrated;
    TimeSync_Slew(0, timeSyncPeriodUS, &remaining);
    if (TimeSync_PLLSupported()) {
@@ -652,14 +660,14 @@ TimeSyncDoSync(Bool slewCorrection,
              TimeSync_IsGuestSyncServiceRunning()) {
             if (data->guestResyncTimer == NULL) {
                g_debug("Guest resync: stepping time.\n");
-               if (!TimeSync_DoGuestResync()) {
+               ASSERT(data->ctx != NULL);
+               if (!TimeSync_DoGuestResync(data->ctx)) {
                   g_warning("Guest resync operation failed.\n");
                   return TimeSyncDoSync(data->slewCorrection,
                                         TIMESYNC_STEP_NORESYNC,
                                         allowBackwardSync, data);
                }
                if (data->guestResyncTimeout > 0) {
-                  ASSERT(data->ctx != NULL);
                   data->guestResyncTimer =
                      g_timeout_source_new(data->guestResyncTimeout);
                   VMTOOLSAPP_ATTACH_SOURCE(data->ctx, data->guestResyncTimer,
